@@ -1,5 +1,5 @@
 /* nobleNote, a note taking application
- * Copyright (C) 2012 Christian Metscher <hakaishi@web.de>,
+ * Copyright (C) 2019 Christian Metscher <hakaishi@web.de>,
                       Fabian Deuchler <Taiko000@gmail.com>
 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -37,33 +37,37 @@
 #include "fileiconprovider.h"
 #include "textsearchtoolbar.h"
 #include "backup.h"
-#include "flickcharm.h"
 #include "noteimporter.h"
 #include <QTextStream>
 #include <QFile>
 #include <QModelIndex>
 #include <QInputDialog>
 #include <QMouseEvent>
-#include <QDesktopServices>
-#include <QSettings>
 #include <QMessageBox>
 #include <QFileIconProvider>
 #include <QList>
 #include <QPushButton>
 #if QT_VERSION >= 0x050000
 #include <QtConcurrent/QtConcurrentMap>
+#include <QScroller>
+#include <QStandardPaths>
 #else
-#include  <QtConcurrentMap>
+#include <QtConcurrentMap>
+#include <QDesktopServices>
 #endif
 
 MainWindow::MainWindow()
 {
      setupUi(this);
 
+     fileWatcher = new QFileSystemWatcher(this);
+
+     createAndUpdateRecent();
+
    //TrayIcon
      QIcon icon = QIcon(":nobleNote");
 
-     minimizeRestoreAction = new QAction(tr("&Minimize"),this);
+     minimizeRestoreAction = new QAction(tr("&Restore"),this);
      quit_action = new QAction(tr("&Quit"),this);
 
 #ifndef NO_SYSTEM_TRAY_ICON
@@ -126,9 +130,6 @@ MainWindow::MainWindow()
      noteFSModel->setReadOnly(false);
      noteFSModel->setIconProvider(noteIconProvider);
 
-     //foreach(QString str, noteFSModel->mimeTypes())
-     //    qDebug() << "mime types: " << str;
-
      findNoteModel = new FindFileModel(this);
 
      noteModel = new FindFileSystemModel(this);
@@ -142,15 +143,13 @@ MainWindow::MainWindow()
      folderView->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
      noteView->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
 
-     flickCharm = new FlickCharm(this);
+     const QList<QListView*> listViews = QList<QListView*>() << folderView << noteView;
 
-     QList<QListView*> listViews;
-     listViews << folderView << noteView;
-     foreach(QListView* list, listViews) // add drag drop options
+     for(QListView* list : listViews) // add drag drop options
      {
         if(QSettings().value("kinetic_scrolling", false).toBool())
         {
-            flickCharm->activateOn(list);// enable kinetic scrolling for touchscreens
+            QScroller::grabGesture(list->viewport(), QScroller::LeftMouseButtonGesture);
         }
         list->setContextMenuPolicy(Qt::CustomContextMenu);
         //list->setSelectionMode(QAbstractItemView::SingleSelection); // single item can be draged or droped
@@ -186,6 +185,7 @@ MainWindow::MainWindow()
      //connect(searchName, SIGNAL(textChanged(const QString)), this, SLOT(find()));
      connect(searchText, SIGNAL(textChanged(const QString)), this, SLOT(find()));
      connect(noteFSModel,SIGNAL(fileRenamed(QString,QString,QString)),this,SLOT(noteRenameFinished(QString,QString,QString)));
+     connect(fileWatcher, SIGNAL(fileChanged(const QString)), this, SLOT(createAndUpdateRecent()));
      //     connect(folderList, SIGNAL(clicked(const QModelIndex &)), this,
      //       SLOT(setCurrentFolder(const QModelIndex &)));
      //     connect(folderList,SIGNAL(activated(QModelIndex)), this,
@@ -225,9 +225,12 @@ MainWindow::MainWindow()
      //connect(actionHistory, SIGNAL(triggered()), this, SLOT(showHistory()));
 
      connect(actionConfigure, SIGNAL(triggered()), this, SLOT(showPreferences()));
-     connect(actionAbout,SIGNAL(triggered()),this,SLOT(about()));
+     connect(actionAbout, SIGNAL(triggered()), this, SLOT(about()));
 
      connect(searchText, SIGNAL(sendCleared()), this, SLOT(selectFolder()));
+
+     connect(folderFSModel, SIGNAL(droppedAFile()), this, SLOT(createAndUpdateRecent())); //moving a note with
+                                                                 //drag and drop removes it from the recent files
 }
 
 MainWindow::~MainWindow()
@@ -243,38 +246,60 @@ void MainWindow::selectFolder()
      folderActivated(folderView->selectionModel()->selectedIndexes().first());
 }
 
-void MainWindow::writeBackupDirPath()
+void MainWindow::writeBackupDirPath() //generates a backup path according to OS and portability
 {
-     QString suffix = QSettings().value("root_path").toString();
-    #ifdef Q_OS_WIN32
-     suffix.replace("\\", "_");
-    #else
-     suffix.replace("/", "_");
-    #endif
+     QString backupPath;
+     QString suffix = "";
 
-   #ifdef Q_OS_WIN32
-     suffix.prepend("_");
-     suffix.remove(":");
+     if(!QSettings().value("isPortable",false).toBool()) //if not portable
+     {
+       suffix = QSettings().value("root_path").toString();
 
-     QString backupPath = QDesktopServices::storageLocation(QDesktopServices::DataLocation);
+      #ifdef Q_OS_WIN32
+       suffix.prepend(QDir::separator()); //we want to seperate "backup" from the rest of the path (see below)
+       suffix.remove(":"); //removing illegal character from path
+      #endif
+
+       suffix.replace(QDir::separator(), "_");
+
+      #if QT_VERSION < 0x050000 && !defined(Q_OS_WIN32)
+       backupPath = QDir::home().absolutePath() +  "/.local/share/" + qApp->applicationName();
+      #elif QT_VERSION < 0x050000
+       backupPath = QDesktopServices::storageLocation(QDesktopServices::DataLocation);
+      #elif QT_VERSION >= 0x050000
+       backupPath = QStandardPaths::standardLocations(QStandardPaths::DataLocation).first();
+      #endif
 
      // reduce extraordinary long path, replaces .../nobleNote/nobleNote/... with .../nobleNote/...
-     if(backupPath.contains("\\" + qApp->organizationName() + "\\" + qApp->applicationName()))
-     {
-         backupPath.replace("\\" + qApp->organizationName() + "\\" + qApp->applicationName(),"\\" +qApp->applicationName());
-     }
-   #else
-     QString backupPath = QDir::homePath() + "/.local/share/" + qApp->applicationName();
-   #endif
 
-     QSettings().setValue("backup_dir_path", backupPath + "/backups" + suffix);
+        QString doubleNobleNote =
+                (QDir::separator() + qApp->organizationName() + QDir::separator() + qApp->applicationName());
+        QString singleNobleNote = QDir::separator() +qApp->applicationName();
+
+        // use unified separators, because QDir::separator() is platform native (\ on Windows)
+        // but QStandardPaths uses / on Windows and Linux
+        backupPath = QDir().toNativeSeparators(backupPath);
+        doubleNobleNote = QDir().toNativeSeparators(doubleNobleNote);
+        singleNobleNote = QDir().toNativeSeparators(singleNobleNote);
+
+       if(backupPath.contains(doubleNobleNote))
+         backupPath.replace(doubleNobleNote,singleNobleNote);
+     }
+     else //if portable
+       backupPath = qApp->applicationDirPath();
+
+     QSettings().setValue("backup_dir_path", backupPath + QDir::separator() + "backups" + suffix);
+     //note that suffix will be "" if portable
 }
 
 void MainWindow::changeRootIndex(){
      if(!openNotes.isEmpty()){
-        foreach(QWidget *note, openNotes)
+        for(QWidget *note : openNotes)
             if(note)
+            {
                 note->close();
+            }
+
         openNotes.clear();
      }
      writeBackupDirPath();
@@ -296,16 +321,16 @@ void MainWindow::makeStandardPaths(){
      {
          QString defaultDirName = tr("default");
          QDir(QSettings().value("root_path").toString()).mkdir(defaultDirName);
-         noteView->setRootIndex(noteModel->setRootPath(QSettings().value("root_path").toString() + "/" + defaultDirName)); // set default dir as current note folder
+         noteView->setRootIndex(noteModel->setRootPath(QSettings().value("root_path").toString() + QDir::separator() + defaultDirName)); // set default dir as current note folder
      }
      else
-         noteView->setRootIndex(noteModel->setRootPath(QSettings().value("root_path").toString() + "/" + dirList.first())); // dirs exist, set first dir as current note folder
+         noteView->setRootIndex(noteModel->setRootPath(QSettings().value("root_path").toString() + QDir::separator() + dirList.first())); // dirs exist, set first dir as current note folder
 
      //Select the first folder
      if(!dirList.isEmpty())
      {
           folderView->selectionModel()->select(folderModel->index(QSettings().value(
-               "root_path").toString() + "/" + dirList.first()),QItemSelectionModel::Select);
+               "root_path").toString() + QDir::separator() + dirList.first()),QItemSelectionModel::Select);
      }
 }
 
@@ -323,6 +348,7 @@ void MainWindow::showPreferences()
           pref = new Preferences(this);
           connect(pref, SIGNAL(pathChanged()), this, SLOT(changeRootIndex()));
           connect(pref,SIGNAL(kineticScrollingEnabledChanged(bool)),this,SLOT(setKineticScrollingEnabled(bool)));
+          connect(pref, SIGNAL(recentCountChanged()), this, SLOT(createAndUpdateRecent()));
      }
      pref->show();
 }
@@ -349,6 +375,19 @@ void MainWindow::find()
 
 void MainWindow::folderRenameFinished(QWidget *editor, QAbstractItemDelegate::EndEditHint hint)
 {
+     QString newNotebookName = folderView->selectionModel()->selectedRows().first().data(Qt::DisplayRole).toString();
+     QStringList recent = QSettings().value("Recent_notes").toStringList();
+
+     for(int i = 0; i < recent.size(); i++)
+     {
+          QString fileName = QFileInfo(recent[i]).fileName();
+          QString strippedPath = recent[i];
+          strippedPath.chop(fileName.size() + getToBerenamedNotebook.size() +1);
+          if(!strippedPath.contains(strippedPath + getToBerenamedNotebook))
+            recent[i] = strippedPath + newNotebookName + QDir::separator() + fileName;
+     }
+     QSettings().setValue("Recent_notes", recent);
+
      if(folderView->selectionModel()->selectedRows().isEmpty())
        return;
      Q_UNUSED(editor);
@@ -369,14 +408,21 @@ void MainWindow::folderRenameFinished(QWidget *editor, QAbstractItemDelegate::En
 void MainWindow::noteRenameFinished(const QString & path, const QString & oldName, const QString & newName)
 {
      QString filePath = noteModel->filePath(noteView->currentIndex());
-     Note * w = noteWindow(path + "/" + oldName);
+     Note * w = noteWindow(path + QDir::separator() + oldName);
      if(w)
-        w->setWindowTitle(QFileInfo(path + "/" + newName).baseName());
+        w->setWindowTitle(QFileInfo(path + QDir::separator() + newName).baseName());
 
      noteView->model()->sort(0);
      noteView->setCurrentIndex(noteModel->index(filePath));
 
      noteView->scrollTo(noteView->selectionModel()->selectedRows().first());
+
+     QStringList recent = QSettings().value("Recent_notes").toStringList();
+
+     for(int i = 0; i < recent.size(); i++)
+          if(recent[i].contains(path + QDir::separator() + oldName))
+            recent.replace(i, path + QDir::separator() + newName);
+     QSettings().setValue("Recent_notes", recent);
 }
 
 void MainWindow::folderActivated(const QModelIndex &selected)
@@ -419,14 +465,16 @@ void MainWindow::noteActivated(const QItemSelection &selected, const QItemSelect
 }
 
 #ifndef NO_SYSTEM_TRAY_ICON
-void MainWindow::iconActivated(QSystemTrayIcon::ActivationReason reason){
+void MainWindow::iconActivated(QSystemTrayIcon::ActivationReason reason)
+{
      if(reason == QSystemTrayIcon::Trigger)
        tray_actions();
 }
 #endif
 
 #ifndef NO_SYSTEM_TRAY_ICON
-void MainWindow::tray_actions(){
+void MainWindow::tray_actions()
+{
      if(isMinimized() || isHidden())  //in case that the window is minimized or hidden
        showNormal();
      else
@@ -434,13 +482,32 @@ void MainWindow::tray_actions(){
 }
 #endif
 
-void MainWindow::showEvent(QShowEvent* show_window){
+/*#ifndef NO_SYSTEM_TRAY_ICON
+void MainWindow::updateMyTrayIcon()
+{
+     trayMenu
+}
+#endif*/
+
+void MainWindow::showEvent(QShowEvent* show_window)
+{
+     minimizeRestoreAction->setText(tr("&Minimize"));
      if(QSettings().contains("mainwindow_size"))
        restoreGeometry(QSettings().value("mainwindow_size").toByteArray());
      if(QSettings().contains("splitter"))
        splitter->restoreState(QSettings().value("splitter").toByteArray());
+
+     showOpenNotes();
+
+     QMainWindow::showEvent(show_window);
+}
+
+void MainWindow::showOpenNotes() //do not confuse with "recent"
+{
      if(QSettings().value("open_notes").isValid())
-       foreach(QString path, QSettings().value("open_notes").toStringList())
+     {
+       const QStringList list = QSettings().value("open_notes").toStringList();
+       for(QString path : list)
        {
            // check if the notePath is already used in a open note
           if(!noteIsOpen(path) && QFile(path).exists())
@@ -448,18 +515,20 @@ void MainWindow::showEvent(QShowEvent* show_window){
                Note* note = new Note(path);
                openNotes += note;
                note->setObjectName(path);
-               note->show();
+               note->showAfterLoaded();
           }
        }
-     QMainWindow::showEvent(show_window);
+     }
 }
 
-void MainWindow::hideEvent(QHideEvent* window_hide){
+void MainWindow::hideEvent(QHideEvent* window_hide)
+{
      minimizeRestoreAction->setText(tr("&Restore"));
      QMainWindow::hideEvent(window_hide);
 }
 
-void MainWindow::closeEvent(QCloseEvent* window_close){
+void MainWindow::closeEvent(QCloseEvent* window_close)
+{
      if(QSettings().value("dont_quit_on_close").toBool())
      {
        hide();
@@ -467,8 +536,31 @@ void MainWindow::closeEvent(QCloseEvent* window_close){
      else{
        QSettings().setValue("mainwindow_size", saveGeometry());
        QSettings().setValue("splitter", splitter->saveState());
-       qApp->quit();
+
+       // find widgets that still are saving their settings and let their futuures (worker threads) finish
+       QListIterator<QPointer<QWidget> > it(openNotes);
+       while(it.hasNext())
+       {
+           QPointer<QWidget> ptr = it.next();
+           QWidget * widget = ptr.data();
+           Note * note = qobject_cast<Note*>(widget);
+           if(note != NULL)
+           {
+               // calls close event (this is normally not called, because Notes are no childs of this)
+               // TODO check why Notes are no children of this
+
+               note->close();
+               if(!note->future().isFinished())
+                   note->future().waitForFinished();
+
+               // close() calls the closeEvent, this method assumes the user has closed the window
+               // so we manually have to re-add this note to the list of open notes.
+               Note::addToOpenNoteList(note->noteDescriptor()->filePath());
+           }
+       }
+       qApp->setQuitOnLastWindowClosed(true);
      }
+
      QMainWindow::closeEvent(window_close);
 }
 
@@ -497,8 +589,8 @@ void MainWindow::openNote(const QModelIndex &index /* = new QModelIndex*/){
 }
 
 void MainWindow::openAllNotes(){
-     QList<QModelIndex> indexes = noteView->selectionModel()->selectedRows();
-     foreach(QModelIndex ind, indexes)
+     const QList<QModelIndex> indexes = noteView->selectionModel()->selectedRows();
+     for(QModelIndex ind :  indexes)
      {
           if(!ind.isValid()) // default constructed model index
             ind = noteView->currentIndex();
@@ -513,22 +605,74 @@ void MainWindow::openAllNotes(){
           if(noteIsOpen(notePath))
             return;
 
-          Note* note = new Note(notePath);
-          openNotes += note;
-          note->setObjectName(notePath);
-          QStringList savedOpenNoteList;
-          if(QSettings().value("open_notes").isValid())
-            savedOpenNoteList = QSettings().value("open_notes").toStringList();
-          savedOpenNoteList.append(notePath);
-          QSettings().setValue("open_notes",savedOpenNoteList);
-          if(QSettings().value("kinetic_scrolling", false).toBool())
-          {
-              flickCharm->activateOn(note->textEdit());
-          }
-          // only show the searchBar if the note contains the search text
-          if(noteModel->sourceModel() == findNoteModel && note->textEdit()->document()->toPlainText().contains(searchText->text(),Qt::CaseInsensitive))
-             note->setSearchBarText(searchText->text());
-          note->show();
+          openOneNote(notePath);
+     }
+}
+
+void MainWindow::openOneNote(QString path)
+{
+     Note* note = new Note(path);
+     openNotes += note;
+     note->setObjectName(path);
+
+     Note::addToOpenNoteList(path);
+     if(QSettings().value("kinetic_scrolling", false).toBool())
+     {
+         QScroller::grabGesture(note->textEdit()->viewport(), QScroller::LeftMouseButtonGesture);
+     }
+     // only show the searchBar if the note contains the search text
+     if(noteModel->sourceModel() == findNoteModel)
+        note->setSearchBarText(searchText->text());
+     note->showAfterLoaded();
+
+     QStringList recentNoteList = QSettings().value("Recent_notes").toStringList();
+     recentNoteList.removeAll(path); //remove if already present
+     recentNoteList.prepend(path); //prepend to be at the first place
+     while(recentNoteList.size() > QSettings().value("Number_of_recent_Notes",5).toInt())
+       recentNoteList.removeLast(); //make sure to keep the size set in the preferences
+     QSettings().setValue("Recent_notes", recentNoteList);
+     createAndUpdateRecent(); //also calls createRecent
+}
+
+void MainWindow::createAndUpdateRecent()
+{
+     menu_Open_recent->clear();
+     QStringList recentFilePaths = QSettings().value("Recent_notes").toStringList();
+
+     //remove all nonexisting note paths
+     for(int i = 0; i < recentFilePaths.size(); i++)
+        if(!QFile(recentFilePaths[i]).exists())
+          recentFilePaths.removeAll(recentFilePaths[i]);
+
+     //remove all note paths to match the size in the preferences
+     while(recentFilePaths.size() > QSettings().value("Number_of_recent_Notes",5).toInt())
+       recentFilePaths.removeLast();
+
+     fileWatcher->addPaths(recentFilePaths);
+     QSettings().setValue("Recent_notes", recentFilePaths); //save for later
+
+     menu_Open_recent->setDisabled(QSettings().value("Recent_notes").toStringList().isEmpty()); //disable if zero
+
+     QAction *recentAction = 0;
+     for(int i = 0; i < recentFilePaths.size(); i++)
+     {
+          recentAction = new QAction(menu_Open_recent);
+          QString fileName = QFileInfo(recentFilePaths[i]).fileName(); //get file name
+          recentAction->setText(fileName); //displayed name
+          recentAction->setData(recentFilePaths[i]); //add the path as data
+          connect(recentAction, SIGNAL(triggered()), this, SLOT(openRecent())); 
+          menu_Open_recent->addAction(recentAction);
+     }
+}
+
+void MainWindow::openRecent()
+{
+     QAction *action = qobject_cast<QAction *>(sender()); //get QAction that was the sender of the signal
+     if(action){ //if not NULL
+       if(noteIsOpen(action->data().toString()))
+         return;
+       else
+         openOneNote(action->data().toString()); //open note with the path in data of QAction
      }
 }
 
@@ -548,7 +692,7 @@ void MainWindow::openNoteSource()
 
     if(QSettings().value("kinetic_scrolling", false).toBool())
     {
-        flickCharm->activateOn(textEdit);
+        QScroller::grabGesture(textEdit->viewport(), QScroller::LeftMouseButtonGesture);
     }
 
     mainWindow->setCentralWidget(textEdit);
@@ -586,12 +730,12 @@ Note *MainWindow::noteWindow(const QString &filePath)
 }
 
 void MainWindow::newFolder(){
-     QString path = folderModel->rootPath() + "/" + tr("new notebook");
+     QString path = folderModel->rootPath() + QDir::separator() + tr("new notebook");
      int counter = 0;
      while(QDir(path).exists())
      {
          ++counter;
-         path = folderModel->rootPath() + "/" + tr("new notebook (%1)").arg(QString::number(counter));
+         path = folderModel->rootPath() + QDir::separator() + tr("new notebook (%1)").arg(QString::number(counter));
      }
      QModelIndex idx = folderModel->mkdir(folderView->rootIndex(),QDir(path).dirName());
 
@@ -605,12 +749,12 @@ void MainWindow::newFolder(){
 }
 
 void MainWindow::newNote(){
-     QString filePath = noteModel->rootPath() + "/" + tr("new note");
+     QString filePath = noteModel->rootPath() + QDir::separator() + tr("new note");
      int counter = 0;
      while(QFile::exists(filePath))
      {
          ++counter;
-         filePath = noteModel->rootPath() + "/" + tr("new note (%1)").arg(QString::number(counter));
+         filePath = noteModel->rootPath() + QDir::separator() + tr("new note (%1)").arg(QString::number(counter));
      }
 
      QFile file(filePath);
@@ -625,12 +769,16 @@ void MainWindow::newNote(){
          noteView->edit(idx); // 'open' for rename
      }
      noteView->model()->sort(0);
-     noteView->scrollTo(noteView->selectionModel()->selectedRows().first());
+     if(!noteView->selectionModel()->selectedRows().isEmpty())
+     {
+        noteView->scrollTo(noteView->selectionModel()->selectedRows().first());
+     }
 }
 
 void MainWindow::renameFolder(){
      if(!folderView->selectionModel()->selectedRows().isEmpty())
        folderView->edit(folderView->selectionModel()->selectedRows().first());
+     getToBerenamedNotebook = folderView->selectionModel()->selectedRows().first().data(Qt::DisplayRole).toString();
 }
 
 void MainWindow::renameNote(){
@@ -664,10 +812,10 @@ void MainWindow::removeFolder(){
 
         // list all files
         QString path = folderModel->filePath(idx);
-        QStringList fileList = QDir(path).entryList(QDir::NoDotAndDotDot | QDir::Dirs | QDir::Files);
+        const QStringList fileList = QDir(path).entryList(QDir::NoDotAndDotDot | QDir::Dirs | QDir::Files);
 
         QModelIndexList indexes;
-        foreach(const QString & fileName, fileList)
+        for(const QString & fileName : fileList)
         {
             indexes << folderModel->index(QString("%1/%2").arg(path).arg(fileName));
         }
@@ -706,21 +854,32 @@ void MainWindow::removeNote(){
      if(noteView->selectionModel()->selectedRows().isEmpty())
        return;
 
-
-     int numNotes = noteView->selectionModel()->selectedRows().size();
      QString names;
-     foreach(QString name, noteModel->fileNames(noteView->selectionModel()->selectedRows()))
+     const auto list = noteModel->fileNames(noteView->selectionModel()->selectedRows());
+     for(QString name : list)
           names += "\"" + name + "\"\n";
 
-     QString title = tr("Delete Note");
-     QString text = tr("Are you sure you want to move the note %1 to the trash?").arg(names);
+     QString title;
+     QString text;
 
-     if(numNotes > 1)
+     int numNotes = noteView->selectionModel()->selectedRows().size();
+
+     if(noteModel->allSizeZero(noteView->selectionModel()->selectedRows()))
      {
-        title = tr("Delete Multiple Notes");
-        text =  tr("Are you sure you want to move these %1 notes to the trash?\n\n%2").arg(QString::number(numNotes)).arg(names);
+         // zero size files do not go to the trash
+         title = tr("Deleting notes");
+         text = tr("Are you sure you want to permanently delete the selected notes?");
+     }
+     else if(numNotes > 1)
+     {
+         title = tr("Delete Multiple Notes");
+         text =  tr("Are you sure you want to move these %1 notes to the trash?\n\n%2").arg(QString::number(numNotes)).arg(names);
+     }
+     else
+     {
+         title = tr("Delete Note");
+         text = tr("Are you sure you want to move the note %1 to the trash?").arg(names);
      }
-
 
      if(QMessageBox::warning(this,title,text,QMessageBox::Yes | QMessageBox::No) != QMessageBox::Yes)
        return;
@@ -734,22 +893,28 @@ void MainWindow::removeNote(){
 
 void MainWindow::setKineticScrollingEnabled(bool b)
 {
-    QList<QAbstractScrollArea* > widgets;
-    widgets << noteView << folderView;
-    foreach(QWidget * w, openNotes)
+    QList<QAbstractScrollArea* > widgets  = QList<QAbstractScrollArea*>() << noteView << folderView;
+    for(QWidget * w : openNotes)
+    {
         if(Note * note = qobject_cast<Note*>(w))
             widgets << note->textEdit();
+    }
+
     if(b)
     {
-        foreach(QAbstractScrollArea* widget, widgets)
+        for(QAbstractScrollArea* widget : widgets)
             if(widget)
-                flickCharm->activateOn(widget);
+            {
+                QScroller::grabGesture(widget->viewport(), QScroller::LeftMouseButtonGesture);
+            }
     }
     else
     {
-        foreach(QAbstractScrollArea* widget, widgets)
+        for(QAbstractScrollArea* widget : widgets)
             if(widget)
-                flickCharm->deactivateFrom(widget);
+            {
+                QScroller::ungrabGesture(widget->viewport());
+            }
     }
 }
 
@@ -821,8 +986,11 @@ void MainWindow::getCutFiles()
      shortcutNoteList.clear();
 
      if(noteView->hasFocus())
-       foreach(QModelIndex idx, noteView->selectionModel()->selectedRows())
+     {
+         const auto selected = noteView->selectionModel()->selectedRows();
+       for(QModelIndex idx  : selected)
           shortcutNoteList << noteModel->filePath(idx);
+     }
 
      if(!shortcutNoteList.isEmpty())
        action_Paste->setEnabled(true);
@@ -832,6 +1000,10 @@ void MainWindow::getCutFiles()
      noteIconProvider = new FileIconProvider();
      noteIconProvider->setCutFiles(shortcutNoteList);
      noteFSModel->setIconProvider(noteIconProvider);
+
+     const auto selected = noteView->selectionModel()->selectedRows();
+     for(QModelIndex idx : selected)
+       noteView->update(idx);
 }
 
 void MainWindow::pasteFiles()
@@ -840,11 +1012,11 @@ void MainWindow::pasteFiles()
        return;
 
      QString copyErrorFiles;
-     foreach(QString note, shortcutNoteList)
+     for(QString note : shortcutNoteList)
      {
           if(!QFile(note).copy(folderModel->filePath(
              folderView->selectionModel()->selectedRows().first())
-             + "/" + QFileInfo(note).fileName()))
+             + QDir::separator() + QFileInfo(note).fileName()))
                      copyErrorFiles += "\"" + note + "\"\n";
           else QFile(note).remove();
      }
@@ -876,6 +1048,14 @@ void MainWindow::keyPressEvent(QKeyEvent *k){
             delete noteIconProvider;
           noteIconProvider = new FileIconProvider();
           noteFSModel->setIconProvider(noteIconProvider);
+
+          //update noteView's items
+          QSet<QModelIndex> rows; //no duplicates for performance
+          for(int i = 0; noteView->indexAt(QPoint(0,i)).isValid(); i++)
+               rows << noteView->indexAt(QPoint(0,i));
+
+          for(QModelIndex index : rows)
+               noteView->update(index);
      }
 }
 
@@ -888,9 +1068,9 @@ void MainWindow::about()
      QString version = in.readLine();
      versionFile.close();
 
-     QMessageBox::about(this, tr("About ") + QApplication::applicationName(),
+     QMessageBox::about(this, tr("About ") + qApp->applicationName(),
                       tr("<h1>%1 version %2</h1><p><b>%1</b> is a note taking application</p>"
-                   "<p>Copyright (C) 2012 Christian Metscher, Fabian Deuchler</p>"
+                   "<p>Copyright © %3 Christian Metscher</p>"
 
                    "<p>Permission is hereby granted, free of charge,"
                    " to any person obtaining a copy of this software and associated documentation files (the \"Software\"),"
@@ -901,9 +1081,10 @@ void MainWindow::about()
                    "<p>THE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,"
                    " FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,"
                    " WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.</p>"
-                         ).arg(QApplication::applicationName()).arg(version)  //: %1 is the application name, also do not translate the licence text
-#ifdef Q_CC_GNU
+                         ).arg(qApp->applicationName()).arg(version).arg(QDate::currentDate().year())  //: %1 is the application name, also do not translate the licence text
+
+                        // these macros should work for every compiler
                         +   "<p>Build " + QString(__TIME__) + " " + QString(__DATE__)  // build time and date
-#endif
+
                         +   "</p><p>Qt " + QString(QT_VERSION_STR) + "</p>");  // the Qt version this build is linked against
 }
